<?php
/**
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * receive this file, see http://cvs.horde.org/co.php/vilma/LICENSE.
 *
 * $Horde: vilma/lib/Driver.php,v 1.22 2005/01/03 14:35:42 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.22 $
 * @package Vilma
 */

class Vilma_Driver {

    /**
     * A hash containing any parameters for the current driver.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Constructor
     *
     * @param optional array $params  Any parameters needed for this driver.
     */
    function Vilma_Driver($params)
    {
        $this->_params = $params;
    }

    /**
     * Returns all the users sorted by domain and as arrays of each domain.
     *
     * @return array  An array of domains then users for each domain.
     */
    function getAllUsers()
    {
        /* Get all users from backend. */
        $users = &$this->getUsers();

        /* Determine the domain for each user and plug into array by domain. */
        $users_by_domain = array();
        foreach ($users as $user) {
            $domain_name = Vilma::stripDomain($user['user_name']);
            $users_by_domain[$domain_name][] = $user;
        }

        /* Sort by domain. */
        ksort($users_by_domain);
        /* Sort each domain's users by user name. */
        require_once 'Horde/Array.php';
        foreach ($users_by_domain as $key => $val) {
            Horde_Array::arraySort($users_by_domain[$key], 'user_name');
        }

        return $users_by_domain;
    }

    /**
     * Checks if the given domain is below the maximum allowed users.
     *
     * @param string $domain  The domain name to check.
     *
     * @return bool  True if the domain does not have a maximum limit (0) or
     *               current number of users is below the maximum number
     *               allowed. Otherwise false.
     */
    function isBelowMaxUsers($domain)
    {
        /* Get the maximum number of users for this domain. */
        $max_users = $this->getDomainMaxUsers($domain);
        if ($max_users == '0') {
            /* No maximum. */
            return true;
        }

        /* Get the current number of users. */
        $num_users = $this->getDomainNumUsers($domain);

        return ($num_users < $max_users);
    }

    /**
     * Does a series of checks for a given user to determine the status.
     *
     * @param array $user  The user's details in an array as returned by the
     *                     getUser() function.
     *
     * @return array  Either an array of error messages found during the checks
     *                or an array with a single element stating that the user
     *                is ready.
     */
    function getUserStatus($user)
    {
        /* Some needed vars. */
        $no_error = true;
        $status = array();
        $domain_name = Vilma::stripDomain($user['user_name']);
        $user_name = Vilma::stripUser($user['user_name']);

        /* Check if user enabled. */
        if (!$user['user_enabled']) {
            $no_error = false;
            $err_msg = _("User disabled.");
            $status[] = Horde::img('alerts/error.png', $err_msg, 'align="middle"', $GLOBALS['registry']->getImageDir('horde')) . '&nbsp;' . $err_msg;
        }

        /* Check if no virtual emails. */
        if (!count($this->getVirtuals($user['user_name']))) {
            $no_error = false;
            $err_msg = _("No virtual emails set.");
            $status[] = Horde::img('alerts/warning.png', $err_msg, 'align="middle"', $GLOBALS['registry']->getImageDir('horde')) . '&nbsp;' . $err_msg;
        }

        /* Check if mailbox exists. */
        $mailboxes = &Vilma::getMailboxDriver();
        if (is_a($mailboxes, 'PEAR_Error')) {
            $no_error = false;
            $err_msg = $mailboxes->getMessage();
            $status[] = Horde::img('alerts/warning.png', $err_msg, 'align="middle"', $GLOBALS['registry']->getImageDir('horde')) . '&nbsp;' . $err_msg;
        }
        $result = $mailboxes->checkMailbox($user_name, $domain_name);
        if (is_a($result, 'PEAR_Error')) {
            $no_error = false;
            $err_msg = $result->getMessage();
            $status[] = Horde::img('alerts/warning.png', $err_msg, 'align="middle"', $GLOBALS['registry']->getImageDir('horde')) . '&nbsp;' . $err_msg;
        }

        /* TODO: Quota checking would be nice too. */

        /* If no errors have been found output a success message for this
         * user's status. */
        if ($no_error) {
            $msg = _("User ready.");
            $status = array(Horde::img('alerts/success.png', $msg, 'align="middle"', $GLOBALS['registry']->getImageDir('horde')) . '&nbsp;' . $msg);
        }
        return $status;
    }

    /**
     * Attempts to return a concrete Driver instance based on $driver.
     *
     * @param string $driver The type of concrete Driver subclass to return.
     *                       This is based on the storage driver ($driver). The
     *                       code is dynamically included.
     * @param optional array $params A hash containing any additional
     *                               configuration or connection parameters a
     *                               subclass might need.
     *
     * @return object Driver The newly created concrete Driver instance, or
     *                false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }
        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Vilma_Driver_' . $driver;
        if (class_exists($class)) {
            return $vilma = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Vilma_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Vilma_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Vilma_Driver::singleton()
     *
     * @param optional string $driver  The type of concrete Vilma_Driver
     *                                 subclass to return.  The is based on
     *                                 the storage driver ($driver).  The code
     *                                 is dynamically included.
     *
     * @param optional array $params   A hash containing any additional
     *                                 configuration or connection parameters
     *                                 a subclass might need.
     *
     * @return mixed The created concrete Vilma_Driver instance, or false on
     *               error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Vilma_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

    function saveUser(&$info)
    {
        $create = false;
        if (empty($info['user_id'])) {
            $create = true;
        }

        $result = &$this->_saveUser($info);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        if ($create) {
            $mailboxes = &Vilma::getMailboxDriver();
            if (is_a($mailboxes, 'PEAR_Error')) {
                return $mailboxes;
            }

            $result = $mailboxes->createMailbox(Vilma::stripUser($info['user_name']), Vilma::stripDomain($info['user_name']));
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        global $conf;
        if (isset($conf['mta']['auth_update_script']) &&
            !empty($info['password'])) {
            $cmd = sprintf('%s set %s %s 2>&1',
                           $conf['mta']['auth_update_script'],
                           escapeshellarg($info['user_name']),
                           escapeshellarg($info['password']));
            $msg = system($cmd, $ec);
            if ($msg === false) {
                return PEAR::raiseError(_("Error running authentication update script."));
            }
            if ($ec != 0) {
                if (empty($msg)) {
                    $msg = _("Unknown error running authentication update script.");
                }
                return PEAR::raiseError($msg);
            }
        }

        return true;
    }

    function deleteUser($user_id)
    {
        $user = $this->getUser($user_id);
        if (is_a($user, 'PEAR_Error')) {
            return $user;
        }

        $res = $this->_deleteUser($user_id);
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        global $conf;
        if (isset($conf['mta']['auth_update_script'])) {


            $cmd = sprintf('%s disable %s 2>&1',
                           $conf['mta']['auth_update_script'],
                           escapeshellarg($user['user_name']));
            $msg = system($cmd, $ec);
            if ($msg === false) {
                return PEAR::raiseError(_("Error running authentication update script."));
            }
            if ($ec != 0) {
                if (empty($msg)) {
                    $msg = _("Unknown error running authentication update script.");
                }
                return PEAR::raiseError($msg);
            }
        }

    }

    /**
     * Get the user who is the domain admin.
     *
     * @todo  This should be replaced by moving all permissions into Horde
     *        permissions.
     */
    function getDomainAdmin($domain_name)
    {
        $domain = $this->getDomainByName($domain_name);
        if (is_a($domain, 'PEAR_Error')) {
            return $domain;
        }
        return $domain['domain_admin'];
    }

    /**
     * Returns the configured quota for this domain.
     *
     * @param string $domain_name  The name of the domain for which to
     *                             return the quota.
     *
     * @return int  The domain's quota.
     */
    function getDomainQuota($domain_name)
    {
        $domain = $this->getDomainByName($domain_name);
        if (is_a($domain, 'PEAR_Error')) {
            return $domain;
        }
        return $domain['domain_quota'];
    }

    /**
     * Returns the maximum number of users allowed for a given domain.
     *
     * @param string $domain_name  The name of the domain for which to
     *                             return the maximum users.
     *
     * @return integer The maximum number of allowed users or PEAR_Error on
     *                 failure. 
     */
    function getDomainMaxUsers($domain_name)
    {
        $domain = $this->getDomainByName($domain_name);
        if (is_a($domain, 'PEAR_Error')) {
            return $domain;
        }
        return $domain['domain_max_users'];
    }

}
