<?php
/**
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * receive this file, see http://cvs.horde.org/co.php/vilma/LICENSE.
 *
 * $Horde: vilma/lib/Driver/sql.php,v 1.25 2005/01/03 14:35:42 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.25 $
 * @package Vilma
 */

class Vilma_Driver_sql extends Vilma_Driver {

    var $_db;

    function Vilma_Driver_sql($params)
    {
        /* Use default table names if these are not set. */
        if (!isset($params['tables']['domains'])) {
            $params['tables']['domains'] = 'vilma_domains';
        }
        if (!isset($params['tables']['users'])) {
            $params['tables']['users'] = 'vilma_users';
        }
        if (!isset($params['tables']['virtuals'])) {
            $params['tables']['virtuals'] = 'vilma_virtuals';
        }

        parent::Vilma_Driver($params);
        $this->initialise();
    }

    /**
     * Construct an SQL WHERE fragment to filter domains by domain key.
     *
     * @private
     * @param optional string $join         Keyword to join expression to rest
     *                                      of SQL statement (e.g. 'WHERE' or
     *                                      'AND').  Default: 'WHERE'.
     * @return string An SQL fragment.
     */
    function _getDomainKeyFilter($join = 'WHERE')
    {
        if (empty($this->_params['tables']['domainkey'])) {
            return '';
        }
        return sprintf(' %s domain_key = %s', $join, $this->_db->quote($this->_params['tables']['domainkey']));
    }

    /**
     * Construct an SQL WHERE fragment to filter users by domain key.
     *
     * @private
     * @param optional string $join         Keyword to join expression to rest
     *                                      of SQL statement (e.g. 'WHERE' or
     *                                      'AND').  Default: 'WHERE'.
     * @return string An SQL fragment.
     */
    function _getUserKeyFilter($join = 'WHERE')
    {
        if (empty($this->_params['tables']['domainkey'])) {
            return '';
        }
        return sprintf(' %s EXISTS ( SELECT domain_name FROM %s WHERE ' .
                       'user_name LIKE \'%%@\' || domain_name %s )',
                       $join, $this->_params['tables']['domains'],
                       $this->_getDomainKeyFilter('AND'));
    }

    /**
     * Construct an SQL WHERE fragment to filter virtuals by domain key.
     *
     * @private
     * @param optional string $join         Keyword to join expression to rest
     *                                      of SQL statement (e.g. 'WHERE' or
     *                                      'AND').  Default: 'WHERE'.
     * @return string An SQL fragment.
     */
    function _getVirtualKeyFilter($join = 'WHERE')
    {
        if (empty($this->_params['tables']['domainkey'])) {
            return '';
        }
        return sprintf(' %s EXISTS ( SELECT domain_name FROM %s WHERE ' .
                       'virtual_email LIKE \'%%@\' || domain_name %s )',
                       $join, $this->_params['tables']['domains'],
                       $this->_getDomainKeyFilter('AND'));
    }

    /**
     * Gets the list of domains from the backend.
     *
     * @return array  All the domains and their data in an array.
     */
    function getDomains()
    {
        $sql = sprintf('SELECT * FROM %s%s ORDER BY domain_name',
                       $this->_params['tables']['domains'],
                       $this->_getDomainKeyFilter());
        return $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Gets the specified domain information from the backend.
     *
     * @param int $domain_id  The id of the domain to fetch.
     *
     * @return array  The domain's information in an array.
     */
    function getDomain($domain_id)
    {
        $sql = sprintf('SELECT * FROM %s WHERE domain_id = %s%s',
                       $this->_params['tables']['domains'],
                       (int)$domain_id, $this->_getDomainKeyFilter('AND'));
        return $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Given a domain name returns the information from the backend.
     *
     * @param string $name  The name of the domain to fetch.
     *
     * @return array  The domain's information in an array.
     */
    function getDomainByName($domain_name)
    {
        $sql = sprintf('SELECT * FROM %s WHERE domain_name = %s%s',
                       $this->_params['tables']['domains'],
                       $this->_db->quote($domain_name),
                       $this->_getDomainKeyFilter('AND'));
        return $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Returns all available users, if a domain name is passed then limit the
     * list of users only to those users.
     *
     * @param optional string $domain  The name of the domain for which to
     *                                 fetch the users.
     *
     * @return array  The available users and their stored information.
     */
    function getUsers($domain = null)
    {
        /* Put together the SQL statement. */
        if (is_null($domain)) {
            /* Fetch all users. */
            $sql = sprintf('SELECT * FROM %s%s',
                           $this->_params['tables']['users'],
                           $this->_getUserKeyFilter());
        } else {
            /* Fetch only users for a domain. */
            $sql = sprintf('SELECT * FROM %s WHERE user_name LIKE %s%s ORDER BY user_name',
                           $this->_params['tables']['users'],
                           $this->_db->quote('%@' . $domain),
                           $this->_getUserKeyFilter('AND'));
        }
        return $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Returns the user information for a given user id.
     *
     * @param int $user_id  The id of the user for which to fetch information.
     *
     * @return array  The user information.
     */
    function getUser($user_id)
    {
        $sql = sprintf('SELECT * FROM %s WHERE user_id = %s%s',
                       $this->_params['tables']['users'],
                       (int)$user_id, $this->_getUserKeyFilter('AND'));
        return $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Returns available virtual emails.
     *
     * @param string $filter  If passed a domain then return all virtual emails
     *                        for the domain, otherwise if passed a user name
     *                        return all virtual emails for that user.
     *
     * @return array  The available virtual emails.
     */
    function getVirtuals($filter)
    {
        /* Check if filtering only for domain. */
        if (($pos = strpos($filter, '@')) === false) {
            $where = sprintf('virtual_email LIKE %s',
                             $this->_db->quote('%@' . $filter));
        } else {
            $where = sprintf('virtual_destination = %s',
                             $this->_db->quote($filter));
        }

        $sql = sprintf('SELECT * FROM %s WHERE %s%s ORDER BY virtual_destination, virtual_email',
                       $this->_params['tables']['virtuals'],
                       $where, $this->_getVirtualKeyFilter('AND'));
        return $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Returns information for a virtual id.
     *
     * @param int $virtual_id  The virtual id for which to return information.
     *
     * @return array  The virtual email information.
     */
    function getVirtual($virtual_id)
    {
        $sql = sprintf('SELECT * FROM %s WHERE virtual_id = %s%s',
                       $this->_params['tables']['virtuals'],
                       (int)$virtual_id, $this->_getVirtualKeyFilter('AND'));
        $virtual = $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
        $virtual['stripped_email'] = Vilma::stripUser($virtual['virtual_email']);

        return $virtual;
    }

    /**
     * Returns the current number of set up users for a domain.
     *
     * @param string $domain_name  The name of the domain for which to
     *                             get the current number of users.
     *
     * @return int  The current number of users.
     */
    function getDomainNumUsers($domain_name)
    {
        $sql = sprintf('SELECT count(user_name) FROM %s WHERE user_name LIKE %s%s',
                       $this->_params['tables']['users'],
                       $this->_db->quote('%@' . $domain_name),
                       $this->_getUserKeyFilter('AND'));
        return $this->_db->getOne($sql);
    }

    /**
     * Saves a domain to the backend.
     *
     * @param array $info  The domain information to save to the backend.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function saveDomain(&$info)
    {
        require_once 'Horde/SQL.php';

        $record = array('domain_name'       => $info['domain_name'],
                        'domain_transport'  => $info['domain_transport'],
                        'domain_admin'      => $info['domain_admin'],
                        'domain_max_users'  => (int)$info['domain_max_users'],
                        'domain_quota'      => (int)$info['domain_quota']);

        if (empty($info['domain_id'])) {
            $record['domain_id'] = $this->_db->nextId($this->_params['tables']['domains']);
            if (!empty($this->_params['tables']['domainkey'])) {
                $record['domain_key'] = $this->_params['tables']['domainkey'];
            }

            $sql = sprintf('INSERT INTO %s %s;',
                           $this->_params['tables']['domains'],
                           Horde_SQL::insertValues($this->_db, $record));
        } else {
            $sql = sprintf('UPDATE %s SET %s WHERE domain_id = %s%s',
                           $this->_params['tables']['domains'],
                           Horde_SQL::updateValues($this->_db, $record),
                           $info['domain_id'],
                           $this->_getDomainKeyFilter('AND'));
        }

        return $this->_db->query($sql);
    }

    /**
     * Deletes a given domain.
     *
     * @param int $domain_id  The id of the domain to delete.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function deleteDomain($domain_id)
    {
        $domain_record = $this->getDomain($domain_id);
        if (is_a($domain_record, 'PEAR_Error')) {
            return $domain_record;
        }

        $domain_name = $domain_record['domain_name'];

        /* Delete all virtual emails for this domain. */
        $sql = sprintf('DELETE FROM %s WHERE virtual_email LIKE %s',
                       $this->_params['tables']['virtuals'],
                       $this->_db->quote('%@' . $domain_name));
        $delete = $this->_db->query($sql);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        /* Delete all users for this domain. */
        $sql = sprintf('DELETE FROM %s WHERE user_name LIKE %s',
                       $this->_params['tables']['users'],
                       $this->_db->quote('%@' . $domain_name));
        $delete = $this->_db->query($sql);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        /* Finally delete the domain. */
        $sql = sprintf('DELETE FROM %s WHERE domain_id = %s',
                       $this->_params['tables']['domains'],
                       (int)$domain_id);
        return $this->_db->query($sql);
    }

    /**
     * Saves a user to the backend.
     *
     * @param array $info  The user information to save.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function _saveUser($info)
    {
        global $conf;

        require_once 'Horde/SQL.php';

        /* Access check (for domainkey). */
        $res = $this->getDomainByName(Vilma::stripDomain($info['user_name']));
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        $mailboxes = &Vilma::getMailboxDriver();
        if (is_a($mailboxes, 'PEAR_Error')) {
            return $mailboxes;
        }

        if (empty($info['user_id'])) {
            $info['user_id'] = $this->_db->nextId($this->_params['tables']['users']);
            $create = true;
        } else {
            $create = false;
        }

        // Slightly hackish.
        $mail_dir_base = isset($mailboxes->_params['mail_dir_base']) ?
                         $mailboxes->_params['mail_dir_base'] : '?';

        $tuple = array('user_id' =>         (int)$info['user_id'],
                       'user_name' =>       $info['user_name'],
                       'user_full_name' =>  $info['user_full_name'],
                       'user_home_dir' =>   $mail_dir_base,
                       'user_mail_dir' =>   Vilma::stripDomain($info['user_name']) . '/' . Vilma::stripUser($info['user_name']) . '/',
                       'user_mail_quota' => $this->getDomainQuota(Vilma::stripDomain($info['user_name'])) * 1024 * 1024,
                       'user_enabled' =>    (int)$info['user_enabled']);

        // UID and GID are slightly hackish (specific to maildrop driver), too
        if (!isset($mailboxes->_params['uid'])) {
            $tuple['user_uid'] = -1;
        } else {
            $tuple['user_uid'] = $mailboxes->_params['uid'];
        }
        if (!isset($mailboxes->_params['gid'])) {
            $tuple['user_gid'] = -1;
        } else {
            $tuple['user_gid'] = $mailboxes->_params['gid'];
        }

        if (isset($info['password'])) {
            $tuple['user_clear'] = $info['password'];
            $tuple['user_crypt'] = crypt($info['password'],
                                         substr($info['password'], 0, 2));
        } elseif (!$create) {
            return PEAR::raiseError(_("Password must be supplied when creating a new user."));
        }
                       
        if ($create) {
            $sql = sprintf("INSERT INTO %s %s;",
                           $this->_params['tables']['users'],
                           Horde_SQL::insertValues($this->_db, $tuple));
        } else {
            $sql = sprintf("UPDATE %s SET %s WHERE user_id = %d;",
                           $this->_params['tables']['users'],
                           Horde_SQL::updateValues($this->_db, $tuple),
                           (int)$info['user_id']);
        }

        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }
        return true;
    }

    /**
     * Deletes a requested user.
     *
     * @param int $user_id  The id of the user to delete.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function _deleteUser($user_id)
    {
        $user = $this->getUser($user_id);
        if (is_a($user, 'PEAR_Error')) {
            return $user;
        }

        /* Delete all virtual emails for this user. */
        $sql = sprintf('DELETE FROM %s WHERE virtual_destination = %s',
                       $this->_params['tables']['virtuals'],
                       $this->_db->quote($user['user_name']));
        $delete = $this->_db->query($sql);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        /* Delete the actual user. */
        $sql = sprintf('DELETE FROM %s WHERE user_id = %s',
                       $this->_params['tables']['users'],
                       (int)$user_id);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $mailboxes = &Vilma::getMailboxDriver();
        if (is_a($mailboxes, 'PEAR_Error')) {
            return $mailboxes;
        }

        return $mailboxes->deleteMailbox(Vilma::stripUser($user['user_name']),
                                         Vilma::stripDomain($user['user_name']));
    }

    /**
     * Saves virtual email address to the backend.
     *
     * @param array $info     The virtual email data.
     * @param string $domain  The name of the domain for this virtual email.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function saveVirtual(&$info, $domain)
    {
        /* Access check (for domainkey) */
        $res = $this->getDomainByName($domain);
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        if (empty($info['virtual_id'])) {
            $info['virtual_id'] = $this->_db->nextId($this->_params['tables']['virtuals']);
            $sql_sprintf = 'INSERT INTO %s (virtual_email, virtual_destination, virtual_id) VALUES (%s, %s, %s)';
        } else {
            $sql_sprintf = 'UPDATE %s SET virtual_email = %s, virtual_destination = %s WHERE virtual_id = %s';
        }
            $sql = sprintf($sql_sprintf,
                               $this->_params['tables']['virtuals'],
                               $this->_db->quote($info['stripped_email'] . '@' . $domain),
                               $this->_db->quote($info['virtual_destination']),
                               $info['virtual_id']);
        return $this->_db->query($sql);
    }

    /**
     * Deletes a virtual email.
     *
     * @param int $virtual_id  The id of the virtual email to delete.
     */
    function deleteVirtual($virtual_id)
    {
        $sql = sprintf('DELETE FROM %s WHERE virtual_id = %s%s',
                       $this->_params['tables']['virtuals'],
                       $virtual_id, $this->_getVirtualKeyFilter('AND'));
        return $this->_db->query($sql);
    }

    /**
     * Initialise this backend, connect to the SQL database.
     *
     * @return mixed  True on success or PEAR error otherwise.
     */
    function initialise()
    {
        global $registry;

        Horde::assertDriverConfig($this->_params, 'storage',
            array('phptype', 'hostspec', 'username', 'database'));

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        return true;
    }

}
